import get from 'lodash.get';
import { ValidationError } from 'yup';
import { IAllForms } from './models';
import {
  EFieldTypes,
  EnumType,
  IFormFieldBase,
  IFormGroupBase,
  IValidationError,
  IValidationErrors,
} from './modelsCommon';
import { IFormParsed } from './modelsParsed';
import { getFieldError } from './utils';
import { parseFormV1 } from './utilsV1';
import { parseFormV2 } from './utilsV2';

/**
 * @param jsonSchema - jsonSchema validation object
 * @param t - i18n translate function
 * @returns {any}
 * @description getErrorMessages function returns an object with the error messages for each field in the form based on the jsonSchema
 */
export const getErrorMessages = (jsonSchema: any, t?: (params: any) => void) => {
  const errMessages: any = {};

  Object.keys(jsonSchema.properties).forEach((property) => {
    errMessages[property] = {};
    Object.keys(jsonSchema.properties[property]).forEach((key) => {
      const value = jsonSchema.properties[property][key];
      errMessages[property][key] = getFieldError(key, value, t);
      if (Object.keys(jsonSchema).includes(key)) {
        errMessages[property].required = getFieldError('required', null, t);
      }
    });
  });

  return errMessages;
};

/**
 * @param translateKey
 * @param param
 * @returns {IValidationError}
 * @description createErrorObject function returns an object with the error message
 * for a field in the form in a proper format
 * @example createErrorObject('required', { param: 'email' }) => { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: { param: 'email' } }
 */
export const createErrorObject = (
  translateKey: string,
  param?: { [key: string]: any },
): IValidationError => ({
  translationKey: translateKey,
  fallbackLanguageTranslation: translateKey,
  meta: param || {},
});

export function isFormGroup<T extends EnumType>(
  toBeDetermined: IFormFieldBase<T> | IFormGroupBase<T>,
): toBeDetermined is IFormGroupBase<T> {
  return !!(toBeDetermined as IFormGroupBase<T>).children;
}

/**
 * @param stringifiedFunction
 * @returns {Function}
 * @description functionFromString returns a function from a stringified function
 * @example functionFromString('/Function((values) => {return values.medicalConditions.toString() === 'true'})/')
 * => (values) => values.medicalConditions.toString() === 'true'
 */
export const functionFromString = (stringifiedFunction: string): ((props: any) => boolean) => {
  try {
    const substr = stringifiedFunction.substring(10, stringifiedFunction.length - 2);
    return (0, eval)('(' + substr + ')');
  } catch (e) {
    console.log(
      'Error parsing custom validation function in functionFromString. This issue wont cause an error, current validation was skipped',
      e,
    );
    return () => false;
  }
};

/**
 * @param validationError - yup validation error
 * @param initialErrors - initial errors object to be merged with the yup validation errors
 * @returns {IValidationErrors} - returns an object with the errors for each field in the form
 * @description formatYupValidationErrors function returns an object with the errors for each field in the form
 * based on the yup validation error formatting to be used in the GenericForm component
 * @example formatYupValidationErrors(yupValidationError, { email: { property: 'email', constraints: { required: { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: {} } } } })
 */
export const formatYupValidationErrors = <T extends EnumType>(
  validationError: ValidationError,
  initialErrors: IValidationErrors<T> = {},
): IValidationErrors<T> => {
  const errors: IValidationErrors<T> = { ...initialErrors };

  for (const propertyValidationError of validationError.inner) {
    let errorType = propertyValidationError.type;

    if (!errorType && propertyValidationError.params?.regex) {
      if (!propertyValidationError.params?.regex) {
        continue;
      }

      errorType = 'pattern';
    }

    if (!get(errors, `${propertyValidationError.path}`)) {
      errors[propertyValidationError.path as T] = {
        property: propertyValidationError.path as T,
        constraints: {},
      };
    }

    errors[propertyValidationError.path as T]!.constraints[errorType as string] = {
      ...getFieldError(
        errorType as string,
        get(propertyValidationError, 'path', null),
        undefined,
        get(propertyValidationError.params, `${propertyValidationError.type}`),
      ),
      fallbackLanguageTranslation: propertyValidationError.message,
    };
  }

  return errors;
};

/**
 * @param children
 * @param type - field type
 * @param acc - accumulator, used for recursion
 * @returns {string[]} - array of field names
 * @description getFieldNamesByType function returns an array of field names based on the field type
 * @example getFieldNamesByType(form, EFieldTypes.TEXT) => ['firstName', 'lastName']
 */
export const getFieldNamesByType = <T extends EnumType>(
  children: (IFormFieldBase<T> | IFormGroupBase<T>)[],
  type: EFieldTypes,
  acc: T[] = [],
): T[] => {
  return children.reduce((acc: T[], child) => {
    if (isFormGroup(child)) {
      return getFieldNamesByType(child.children, type, acc);
    }

    if (child.type === type) {
      acc.push(child.fieldName as T);
    }

    return acc;
  }, acc);
};

export const getAllFieldNames = <T extends EnumType>(
  children: (IFormFieldBase<T> | IFormGroupBase<T>)[],
  acc: T[] = [],
): T[] => {
  return children.reduce((acc: T[], child) => {
    if (isFormGroup(child)) {
      return getAllFieldNames(child.children, acc);
    }
    acc.push(child.fieldName as T);
    return acc;
  }, acc);
};

export const flattenValues = <T extends EnumType>(obj: { [key in T]: any }): any => {
  const flattenedValues: { [key in T]: any } = { ...obj };
  const keys = Object.keys(obj) as T[];

  keys.forEach((k) => {
    if (typeof obj[k] === 'object') {
      flattenedValues[k] = obj[k].value;
    }
  });

  return flattenedValues;
};

export const valuesToDBFormat = <T extends EnumType>(obj: { [key in T]: any }): any => {
  const dbValues: { [key in T]: any } = { ...obj };
  const keys = Object.keys(obj) as T[];

  keys.forEach((k) => {
    if (typeof obj[k] !== 'object') {
      dbValues[k] = { value: obj[k] };
    }
  });

  return dbValues;
};
/**
 * @param obj - object to trim
 * @returns {any} - object with trimmed values
 * @description trimValuesInObject function returns an object with trimmed values
 */
export const trimValuesInObject = <T extends EnumType>(obj: { [key in T]: any }): any => {
  const trimmedValues: { [key in T]: any } = { ...obj };
  const keys = Object.keys(obj) as T[];

  keys.forEach(
    (k) =>
      (obj[k] =
        typeof obj[k] == 'string'
          ? (trimmedValues[k] = String(obj[k]).trim())
          : (trimmedValues[k] = obj[k])),
  );
  return trimmedValues;
};

export const parseForm = <T extends EnumType>(form: IAllForms<T>): IFormParsed<T> => {
  if (!form?.formatVersion) {
    throw new Error('formatVersion is required');
  }
  switch (form.formatVersion) {
    case 1:
      return parseFormV1(form);
    case 2:
      return parseFormV2(form);
    default:
      throw new Error('The parsing function for this version of forms is not implemented');
  }
};
