export enum EFieldTypes {
  TEXT = 'text',
  NUMBER = 'number',
  SWITCH = 'switch',
  DATE = 'date',
  DATE_RANGE = 'dateRange',
  AUTOCOMPLETE = 'autoComplete',
  FILE = 'file',
  ARRAY = 'array',
  MOBILE_PHONE = 'mobilePhone',
  MULTIPLE_AUTOCOMPLETE = 'autoCompleteMultiple',
  RADIOBUTTON = 'radioButton',
  EMPTY = 'empty',
  CHECKBOX = 'checkbox',
  RADIOBUTTON_MATRIX = 'radioButtonMatrix',
  RATING = 'rating',
}

export type GridSize = 1 | 2 | 3 | 4 | 6 | 8 | 10 | 12 | 'auto';

export interface RegularBreakpoints {
  lg?: GridSize;
  md?: GridSize;
  sm?: GridSize;
  xl?: GridSize;
  xs?: GridSize;
}

export interface IOption {
  id: number;
  label: string;
  value: any;
}

export type EnumType = string | number | symbol;

// ------------------------------ BASE FORM CHILDREN INTERFACES ------------------------------ //
export interface IFormFieldBase<T extends EnumType> {
  type: EFieldTypes;
  fieldName: T;
  initialValue?: any;
}

export interface IFormGroupBase<T extends EnumType> {
  title?: string;
  subtitle?: string;
  children: Array<IFormFieldBase<T> | IFormFieldBase<T>>;
}

// ------------------------------ CROSS FIELD CONDITIONS INTERFACES ------------------------------ //
export type ICrossFieldConditionsParams = {
  disabledCondition?: string;
  renderCondition?: string;
};

// ------------------------------ CROSS FIELD VALIDATION INTERFACES ------------------------------ //
export interface ICrossFieldValidationRule {
  errorKey: string;
  testCondition: string;
  meta?: { param: number };
}

export type ICrossFieldValidations<T extends EnumType> = {
  [key in T]?: ICrossFieldValidationRule[];
};

// ------------------------------ JSON SCHEMA INTERFACES ------------------------------ //
// JSON schema property describes validation for each field in the form
export type IJsonSchemaProperty = {
  id?: string;
  type: 'string' | 'boolean' | 'array' | 'number';
  nullable?: boolean;
  maxLength?: number;
  minLength?: number;
  pattern?: string;
  oneOf?: string[];
  items?: {
    type: 'string';
    format: 'date';
    minLength: number;
    required: boolean;
  };
};

// JSON schema interface https://json-schema.org/learn/getting-started-step-by-step
export interface IJsonSchema<T extends EnumType> {
  $schema: string;
  title: string;
  description: string;
  type: string;
  properties: { [key in T]: IJsonSchemaProperty };
  required: Array<T>;
}

// ------------------------------ OUTPUTS ------------------------------ //
// Validation error DTO interface
export interface IValidationError {
  translationKey: string;
  fallbackLanguageTranslation: string;
  meta: {
    [key: string]: any;
  };
}

// Validation error field DTO interface
export interface IValidationErrorField<T extends EnumType> {
  property: T;
  constraints: {
    [key: string]: IValidationError;
  };
}

// Form validation errors DTO interface
export type IValidationErrors<T extends EnumType> = {
  [key in T]?: IValidationErrorField<T>;
};
